//*******************************************************************************
//  MSP-430-Day II Demo - DMA0 ADC12 A10 Block Transfer to Flash, TBCCR1, DCO 
//
//  Description; A 0x20 word block of data is sampled and recorded into Flash 
//  from the ADC12 channel 10 the integrated temperature sensor using the 
//  Record() function. Timer_B CCR1 begins the ADC12 sample period, CCR0 the hold
//  and conversion start. Timer_B operates in the up mode with CCR0 defining the
//  sample period. DMA0 will automatically transfer each ADC12 conversion code 
//  to memory when complete. At the end of the recording block, DMA0 will issue 
//  an interrupt existing the function.
//  In this example the ADC12 sample period is 200 SMCLK cycles, and Flash
//  programming requires 105 cycles.
//  In the example the Flash block use to record the ADC12 data begins at 
//  0x1080 and the Record() function is called only only to avoide accidental
//  over stressing of the Flash durring demonstration. 
//  P1.0 is toggled durring DMA transfer only for demonstration purposes.
//  ACLK = 32768, MCLK = SMCLK = default DCO =1048576
//  //*An external watch crystal on XIN XOUT is required for ACLK*//	  
//  //*MSP430FG439 Device Required*//
//
//               MSP430FG439
//            -----------------
//        /|\|              XIN|-  
//         | |                 | 32KHz
//         --|RST          XOUT|-
//           |                 |
//           |A10          P1.0|-->LED
//
//  M.Buccini
//  Texas Instruments, Inc
//  February 2004
//  Updated for IAR Embedded Workbench Version: 2.21B
//*******************************************************************************

#include <msp430xG43x.h>

void Record(void); 

void main(void)
{
  unsigned int i;

  WDTCTL = WDTPW + WDTHOLD;                 // Stop watchdog
  FLL_CTL0 |= XCAP14PF;                     // Set load capacitance
  for (i = 0xFFFF; i > 0; i--);             // Time of FLL to settle
  FCTL2 = FWKEY + FSSEL_2 + FN1 + FN0;      // Clk src = SMCLK / 3 = 350KHz
  P1DIR |= 0x01;                            // P1.0 = output

  Record();                             
  _NOP();                                   // << SET BREAKPOINT HERE
}

// Record ADC12 A10 channel data and store in Flash using DMA0
void Record(void)
{
  unsigned int i;

  // unlock and erase Flash memory
  FCTL1 = FWKEY + ERASE;                    // Set Erase bit
  FCTL3 = FWKEY;                            // Clear Lock bit
  *(unsigned int *)0x1080 = 0;              // Dummy write to erase Flash segment

  // setup modules
  ADC12MCTL0 = 0x01A;
  ADC12IFG = 0x00;                           
  ADC12CTL1 = SHS_3 + CONSEQ_2;             // S&H TB.OUT1, rep. single chan
  ADC12CTL0 = REF2_5V + REFON + ADC12ON + ENC;      // VRef ADC12 on, enabled
  for (i = 0xFFF; i > 0; i--);              // Time VRef to settle

  TBCCR0 = 200;                             // Init TBCCR0 w/ sample prd
  TBCCR1 = 200 - 30;                        // Trigger for ADC12 SC
  TBCCTL1 = OUTMOD_7;                       // Reset OUT1 on EQU1, set on EQU0

  DMA0SA = ADC12MEM0_;                      // Src address = ADC12 module
  DMA0DA = 0x1080;                          // Dst address = Flash memory
  DMA0SZ = 0x20;                            // Size in words
  DMACTL0 = DMA0TSEL_6;                     // ADC12IFGx triggers DMA0
  DMACTL1 = DMAONFETCH;                     // Required for proper Flash Write
  DMA0CTL = DMADSTINCR_3 + DMAIE + DMAEN;   // Config

  // start recording and enter LPM0
  P1OUT |= 0x01;                            // Set P1.0 (LED On)
  TBCTL = TBSSEL_2+ MC_1 + TBCLR;           // SMCLK, clear TBR, up mode
  FCTL1 = FWKEY + WRT;                      // Enable Flash write for recording
  _BIS_SR(LPM0_bits + GIE);                 // Enter LPM0 w/ interrupts
  
  // deactivate Flash memory write access
  FCTL1 = FWKEY;                            // Disable Flash write
  FCTL3 = FWKEY + LOCK;                     // Lock Flash memory     

  // power-down MSP430 modules
  ADC12CTL1 &= ~CONSEQ_2;                   // Stop conversion immediately
  ADC12CTL0 &= ~ENC;                        // Disable ADC12 conversion
  ADC12CTL0 = 0;                            // Switch off ADC12 & ref voltage
  TBCTL = 0;                                // Disable Timer_B
  P1OUT &= ~0x01;                           // Clear P1.0 (LED Off)
}
  
// DMA interrupt service routine

interrupt[DAC12_DMA_VECTOR] void DACDMA_ISR (void)
{
  DMA0CTL &= ~DMAIFG;                       // Clear DMA0 interrupt flag
  _BIC_SR_IRQ(LPM0_bits);                   // Exit LPM0 on reti
}
